/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __DT_RT5651_H
#define __DT_RT5651_H

#define RT5651_JD_NULL			0
#define RT5651_JD1_1			1
#define RT5651_JD1_2			2
#define RT5651_JD2			3

#define RT5651_OVCD_SF_0P5		0
#define RT5651_OVCD_SF_0P75		1
#define RT5651_OVCD_SF_1P0		2
#define RT5651_OVCD_SF_1P5		3

#endif /* __DT_RT5651_H */
