/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * mt9v022 sensor
 */

#ifndef __MT9V022_H__
#define __MT9V022_H__

struct mt9v022_platform_data {
	unsigned short y_skip_top;	/* Lines to skip at the top */
};

#endif
