/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (C) 2022-2023 StarFive Technology Co., Ltd.
 * Author: Walker Chen <walker.chen@starfivetech.com>
 */
#ifndef __DT_BINDINGS_POWER_JH7110_POWER_H__
#define __DT_BINDINGS_POWER_JH7110_POWER_H__

#define JH7110_PD_SYSTOP	0
#define JH7110_PD_CPU		1
#define JH7110_PD_GPUA		2
#define JH7110_PD_VDEC		3
#define JH7110_PD_VOUT		4
#define JH7110_PD_ISP		5
#define JH7110_PD_VENC		6

/* AON Power Domain */
#define JH7110_AON_PD_DPHY_TX	0
#define JH7110_AON_PD_DPHY_RX	1

#endif
