/* SPDX-License-Identifier: GPL-2.0-only OR BSD-2-Clause */

#ifndef __DT_BINDINGS_SOC_FSL_TSA_H
#define __DT_BINDINGS_SOC_FSL_TSA_H

#define FSL_CPM_TSA_NU		0	/* Pseuso Cell Id for not used item */
#define FSL_CPM_TSA_SCC2	1
#define FSL_CPM_TSA_SCC3	2
#define FSL_CPM_TSA_SCC4	3
#define FSL_CPM_TSA_SMC1	4
#define FSL_CPM_TSA_SMC2	5

#endif
